//////////////////////////////////////////////////
////////////////Sun Shadowshader//////////////////
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////29.06.2009////////////////by Nils Daumann/////
///////////////www.slindev.com////////////////////
//////////////////////////////////////////////////

//Variables
float4x4 matWorldViewProj;	
float4x4 matWorld;

float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;
float4 vecSkill13;
float4 vecSkill17;

float4x4 matEffect1;
float4x4 matEffect2;
float4x4 matEffect3;
float4x4 matEffect4;

float4 vecSunPos;
float4 vecViewPos;


//Textures
texture entSkin1; //color map
texture mtlSkin1;	//shadow depth map


//Sampler
sampler colorMap = sampler_state
{
   Texture = <entSkin1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
   AddressU = Clamp;
	AddressV = Clamp;
};

sampler depthMap = sampler_state
{
	texture = <mtlSkin1>;
	magfilter = Linear;
	minfilter = Linear;
	mipfilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
//	BorderColor = 0xffffff;
};


//Structs
struct VS_TO_PS
{
	float4 Pos : POSITION;
	float3 Tex : TEXCOORD0;
	float4 PosWorld : TEXCOORD1;
	
	float4 ShadowMapSamplingPos1	: TEXCOORD3;
	float4 ShadowMapSamplingPos2	: TEXCOORD4;
	float4 ShadowMapSamplingPos3	: TEXCOORD5;
	float4 ShadowMapSamplingPos4	: TEXCOORD6;
};


//Vertexshader
VS_TO_PS VS(float4 inPos		: POSITION,
				float2 inTex		: TEXCOORD0)
{
	VS_TO_PS Out = (VS_TO_PS)0;
	
	//Calculate the final vertexposition
	Out.Pos = mul(inPos, matWorldViewProj);
	
	//Get the vertexposition in worldspace
	Out.PosWorld = mul(inPos, matWorld);
	
	//Pass the TexCoords
	Out.Tex.xy = inTex;
	
	Out.Tex.z = saturate(distance(vecViewPos,Out.PosWorld)/vecSkill5.x);
	Out.Tex.z = pow(Out.Tex.z,10);
	Out.Tex.z = 1.0-Out.Tex.z;
	
	Out.ShadowMapSamplingPos1 = mul(Out.PosWorld,matEffect1);
	Out.ShadowMapSamplingPos2 = mul(Out.PosWorld,matEffect2);
	Out.ShadowMapSamplingPos3 = mul(Out.PosWorld,matEffect3);
	Out.ShadowMapSamplingPos4 = mul(Out.PosWorld,matEffect4);
	
	return Out;
}


//Pixelshader
float4 PS(VS_TO_PS In) : COLOR0
{
	float4 Color = tex2D(colorMap,In.Tex.xy);
	clip(Color.a-0.5);

	float3 shadowProjVec[4];
	shadowProjVec[0].xy = In.ShadowMapSamplingPos1.xy/In.ShadowMapSamplingPos1.w;
	shadowProjVec[1].xy = In.ShadowMapSamplingPos2.xy/In.ShadowMapSamplingPos2.w;
	shadowProjVec[2].xy = In.ShadowMapSamplingPos3.xy/In.ShadowMapSamplingPos3.w;
	shadowProjVec[3].xy = In.ShadowMapSamplingPos4.xy/In.ShadowMapSamplingPos4.w;
	shadowProjVec[0].z = In.ShadowMapSamplingPos1.w;
	shadowProjVec[1].z = In.ShadowMapSamplingPos2.w;
	shadowProjVec[2].z = In.ShadowMapSamplingPos3.w;
	shadowProjVec[3].z = In.ShadowMapSamplingPos4.w;
	
	float4 realdepth = distance(vecSkill9.xyz,In.PosWorld.xyz)-vecSkill13.xyzw;///vecSkill17.xyzw;
	float depthnum = 0;
	
	depthnum = lerp(3, 2, shadowProjVec[2].x > 0.0 && shadowProjVec[2].x < 1.0 && shadowProjVec[2].y > 0.0 && shadowProjVec[2].y < 1.0);
	depthnum = lerp(depthnum, 1, shadowProjVec[1].x > 0.0 && shadowProjVec[1].x < 1.0 && shadowProjVec[1].y > 0.0 && shadowProjVec[1].y < 1.0);
	depthnum = lerp(depthnum, 0, shadowProjVec[0].x > 0.0 && shadowProjVec[0].x < 1.0 && shadowProjVec[0].y > 0.0 && shadowProjVec[0].y < 1.0);
	
	float sum = 0;
	float x, y;
	float fShadow;
	
	for (y = -1.5; y <= 1.5; y += 1.5)
	{
		for (x = -1.5; x <= 1.5; x += 1.5)
		{
			sum += tex2D(depthMap, shadowProjVec[depthnum].xy + float2(x, y)*2.5*vecSkill17[depthnum]*vecSkill5.y*vecSkill5.z)[depthnum] > realdepth[depthnum];
		}
	}

//	sum += tex2D(depthMap, shadowProjVec[depthnum].xy)[depthnum] > realdepth[depthnum];
	
	float fLightingFactor = sum/16.0;
	
	return 1.0-(1.0-fLightingFactor)*In.Tex.z;
}


//Technique
technique NormalMapping
{
	pass P0
	{
		alphablendenable = false;
		alphatestenable = false;
		zWriteEnable = true;
		
		// compile shaders
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
}
